@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2008, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2009, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -100,10 +100,9 @@ int vps_stop(vps_handler *h, envid_t veid, vps_param *param, int stop_mode,
  * @param		CT root
  * @return		0 on success
  */
-int vz_chroot(char *root);
+int vz_chroot(const char *root);
 int vz_env_create(vps_handler *h, envid_t veid, vps_res *res,
 	int wait_p[2], int err_p[2], env_create_FN fn, void *data);
 int vz_setluid(envid_t veid);
 int vz_env_create_ioctl(vps_handler *h, envid_t veid, int flags);
-int vps_restart(vps_handler *h, envid_t veid, vps_param *param);
 #endif
