@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2007 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2010, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,10 +25,12 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <string.h>
+#include <grp.h>
 #include <sys/ioctl.h>
 #include <linux/vzcalluser.h>
 #include <sys/personality.h>
 #include <linux/reboot.h>
+#include <sys/mount.h>
 
 #include "vzerror.h"
 #include "res.h"
@@ -41,10 +43,12 @@
 #include "iptables.h"
 #include "readelf.h"
 #include "vzsyscalls.h"
+#include "cpt.h"
 
 #define ENVRETRY	3
 
-static int env_stop(vps_handler *h, envid_t veid, char *root, int stop_mode);
+static int env_stop(vps_handler *h, envid_t veid, const char *root,
+		int stop_mode);
 
 static inline int setluid(uid_t uid)
 {
@@ -57,7 +61,7 @@ static int set_personality(unsigned long mask)
 	unsigned long per;
 
 	per = personality(0xffffffff) | mask;
-	logger(3, 0, "Set personality %#10.8x", per);
+	logger(3, 0, "Set personality %#10.8lx", per);
 	if (personality(per) == -1) {
 		logger(2, errno, "Unable to set personality PER_LINUX32");
 		return  -1;
@@ -84,11 +88,15 @@ int vz_env_create_data_ioctl(vps_handler *h,
 			sleep(1);
 		errcode = ioctl(h->vzfd, VZCTL_ENV_CREATE_DATA, data);
 	} while (errcode < 0 && errno == EBUSY && retry++ < ENVRETRY);
+
+	if (errcode >= 0) {
+		/* Clear supplementary group IDs */
+		setgroups(0, NULL);
 #ifdef  __x86_64__
-	/* Set personality PER_LINUX32 for i386 based VEs */
-	if (errcode >= 0)
+		/* Set personality PER_LINUX32 for i386 based CTs */
 		set_personality32();
 #endif
+	}
 	return errcode;
 }
 
@@ -106,11 +114,14 @@ int vz_env_create_ioctl(vps_handler *h, envid_t veid, int flags)
 			sleep(1);
 		errcode = ioctl(h->vzfd, VZCTL_ENV_CREATE, &env_create);
 	} while (errcode < 0 && errno == EBUSY && retry++ < ENVRETRY);
+	if (errcode >= 0 && (flags & VE_ENTER)) {
+		/* Clear supplementary group IDs */
+		setgroups(0, NULL);
 #ifdef  __x86_64__
-	/* Set personality PER_LINUX32 for i386 based VEs */
-	if (errcode >= 0 && (flags & VE_ENTER))
+		/* Set personality PER_LINUX32 for i386 based CTs */
 		set_personality32();
 #endif
+	}
 	return errcode;
 }
 
@@ -135,9 +146,9 @@ static int reset_std()
 	return stdfd;
 }
 
-/** Allocate and initialize VE handler.
+/** Allocate and initialize CT handler.
  *
- * @param veid		VE id.
+ * @param veid		CT ID.
  * @return		handler or NULL on error.
  */
 vps_handler *vz_open(envid_t veid)
@@ -154,7 +165,7 @@ vps_handler *vz_open(envid_t veid)
 		goto err;
 	}
 	h = calloc(1, sizeof(*h));
-	if (h == NULL) 
+	if (h == NULL)
 		goto err;
 	h->vzfd = vzfd;
 	h->stdfd = stdfd;
@@ -168,8 +179,7 @@ vps_handler *vz_open(envid_t veid)
 	return h;
 
 err:
-	if (h != NULL)
-		free(h);
+	free(h);
 	if (vzfd != -1)
 		close(vzfd);
 	if (stdfd != -1)
@@ -177,9 +187,9 @@ err:
 	return NULL;
 }
 
-/** Close VE handler.
+/** Close CT handler.
  *
- * @param h		VE handler.
+ * @param h		CT handler.
  */
 void vz_close(vps_handler *h)
 {
@@ -191,12 +201,12 @@ void vz_close(vps_handler *h)
 	free(h);
 }
 
-/** Get VE status.
+/** Get CT status.
  *
- * @param h		VE handler.
- * @param veid		VE id.
- * @return		1 - VE is running
- *			0 - VE is stopped.
+ * @param h		CT handler.
+ * @param veid		CT ID.
+ * @return		1 - CT is running
+ *			0 - CT is stopped.
  */
 int vps_is_run(vps_handler *h, envid_t veid)
 {
@@ -216,17 +226,18 @@ int vps_is_run(vps_handler *h, envid_t veid)
 
 /** Change root to specified directory
  *
- * @param		VE root
+ * @param		CT root
  * @return		0 on success
  */
-int vz_chroot(char *root)
+int vz_chroot(const char *root)
 {
 	int i;
 	sigset_t sigset;
 	struct sigaction act;
 
 	if (root == NULL) {
-		logger(-1, 0, "vz_chroot: VE root is not specified");	
+		logger(-1, 0, "vz_chroot: Container root (VE_ROOT) "
+				"not specified");
 		return VZ_VE_ROOT_NOTSET;
 	}
 	if (chdir(root)) {
@@ -261,35 +272,6 @@ int vz_setluid(envid_t veid)
 	return 0;
 }
 
-/*
- * Checks if sysfs needs to be enabled for this VE.
- * Now we do that only for distributions from sysfs_dists.
- * FIXME: provide a generic way to enable/disable sysfs per VE.
-*/
-static int sysfs_required(vps_res *res)
-{
-	static char *sysfs_dists[] = {"opensuse", "suse", "sles", NULL};
-
-	tmpl_param *tmp = &res->tmpl;
-	int len, i;
-	char *name;
-
-	for (i = 0; name = sysfs_dists[i], name != NULL; i++) {
-		len = strlen(name);
-		if (tmp->ostmpl != NULL &&
-			!strncmp(tmp->ostmpl, name, len))
-		{
-			return 1;
-		}
-		if (tmp->dist != NULL &&
-			!strncmp(tmp->dist, name, len))
-		{
-			return 1;
-		}
-	}
-	return 0;
-}
-
 static int _env_create(vps_handler *h, envid_t veid, int wait_p, int err_p,
 	void *data)
 {
@@ -303,7 +285,9 @@ static int _env_create(vps_handler *h, envid_t veid, int wait_p, int err_p,
 	res = (vps_res *) data;
 	memset(&create_param, 0, sizeof(create_param));
 	create_param.iptables_mask = get_ipt_mask(res->env.ipt_mask);
-	logger(3, 0, "Set iptables mask %#10.8x", create_param.iptables_mask);
+	logger(3, 0, "Set iptables mask %#10.8llx",
+			(unsigned long long) create_param.iptables_mask);
+	clean_hardlink_dir("/");
 	if (res->cpu.vcpus != NULL)
 		create_param.total_vcpus = *res->cpu.vcpus;
 	env_create_data.veid = veid;
@@ -314,8 +298,8 @@ static int _env_create(vps_handler *h, envid_t veid, int wait_p, int err_p,
 
 	create_param.feature_mask = res->env.features_mask;
 	create_param.known_features = res->env.features_known;
-	if (!(res->env.features_known & VE_FEATURE_SYSFS) &&
-			sysfs_required(res)) {
+	/* sysfs enabled by default, unless explicitly disabled */
+	if (! res->env.features_known & VE_FEATURE_SYSFS) {
 		create_param.feature_mask |= VE_FEATURE_SYSFS;
 		create_param.known_features |= VE_FEATURE_SYSFS;
 	}
@@ -333,7 +317,7 @@ try:
 			case EINVAL:
 				ret = VZ_ENVCREATE_ERROR;
 				/* Run-time kernel did not understand the
-				 * latest create_parem -- so retry with
+				 * latest create_param -- so retry with
 				 * the old env_create_param structs.
 				 */
 				switch (env_create_data.datalen) {
@@ -368,23 +352,41 @@ try:
 	close(fd);
 
 	mk_reboot_script();
-	if (res->dq.ugidlimit != NULL)
-		mk_quota_link();
+
 	if (res->misc.wait == YES) {
 		if (add_reach_runlevel_mark()) {
 			ret = VZ_WAIT_FAILED;
 			goto env_err;
 		}
 	}
+
+	mount("proc", "/proc", "proc", 0, 0);
+	if (stat_file("/sys"))
+		mount("sysfs", "/sys", "sysfs", 0, 0);
+
+	if (create_param.feature_mask & VE_FEATURE_NFSD) {
+		mount("nfsd", "/proc/fs/nfsd", "nfsd", 0, 0);
+		make_dir("/var/lib/nfs/rpc_pipefs", 1);
+		mount("sunrpc", "/var/lib/nfs/rpc_pipefs", "rpc_pipefs", 0, 0);
+	}
+
+	if (res->dq.ugidlimit != NULL)
+		mk_quota_link();
+
 	/* Close status descriptor to report that
 	 * environment is created.
 	*/
 	close(STDIN_FILENO);
-	/* Now we wait until VE setup will be done
+	/* Now we wait until CT setup will be done
 	   If no error, then start init, otherwise exit.
 	*/
 	if (read(wait_p, &ret, sizeof(ret)) != 0)
 		return 0;
+	if ((fd = open("/dev/null", O_RDWR)) != -1) {
+		dup2(fd, 0);
+		dup2(fd, 1);
+		dup2(fd, 2);
+	}
 	logger(10, 0, "Starting init");
 	execve("/sbin/init", argv, envp);
 	execve("/etc/init", argv, envp);
@@ -510,7 +512,7 @@ static void fix_numiptent(ub_param *ub)
 		return;
 	min_ipt = min_ul(ub->numiptent[0], ub->numiptent[1]);
 	if (min_ipt < MIN_NUMIPTENT) {
-		logger(-1, 0, "Warning: NUMIPTENT %d:%d is less"
+		logger(-1, 0, "Warning: NUMIPTENT %lu:%lu is less"
 			" than minimally allowable value, set to %d:%d",
 			ub->numiptent[0], ub->numiptent[1],
 			MIN_NUMIPTENT, MIN_NUMIPTENT);
@@ -544,28 +546,29 @@ int vps_start_custom(vps_handler *h, envid_t veid, vps_param *param,
 	if (check_var(res->fs.root, "VE_ROOT is not set"))
 		return VZ_VE_ROOT_NOTSET;
 	if (vps_is_run(h, veid)) {
-		logger(-1, 0, "VE is already running");
+		logger(-1, 0, "Container is already running");
 		return VZ_VE_RUNNING;
 	}
 	if ((ret = check_ub(&res->ub)))
 		return ret;
 	dist_name = get_dist_name(&res->tmpl);
+	if (!dist_name)
+		return VZ_SET_NAME_ERROR;
 	ret = read_dist_actions(dist_name, DIST_DIR, &actions);
-	if (dist_name != NULL)
-		free(dist_name);
+	free(dist_name);
 	if (ret)
 		return ret;
-	logger(0, 0, "Starting VE ...");
+	logger(0, 0, "Starting container ...");
 	if (vps_is_mounted(res->fs.root)) {
-		/* if VE is mounted -- umount first, to cleanup mount state */
+		/* if CT is mounted -- umount first, to cleanup mount state */
 		vps_umount(h, veid, res->fs.root, skip);
 	}
 	if (!vps_is_mounted(res->fs.root)) {
 		/* increase quota to perform setup */
-		quouta_inc(&res->dq, 100);	
+		quota_inc(&res->dq, 100);
 		if ((ret = vps_mount(h, veid, &res->fs, &res->dq, skip)))
 			return ret;
-		quouta_inc(&res->dq, -100);
+		quota_inc(&res->dq, -100);
 	}
 	if (pipe(wait_p) < 0) {
 		logger(-1, errno, "Can not create pipe");
@@ -592,7 +595,7 @@ int vps_start_custom(vps_handler *h, envid_t veid, vps_param *param,
 	}
 	if (!(skip & SKIP_ACTION_SCRIPT)) {
 		snprintf(buf, sizeof(buf), VPS_CONF_DIR "%d.%s", veid,
-			START_PREFIX);	
+			START_PREFIX);
 		if (stat_file(buf)) {
 			if (vps_exec_script(h, veid, res->fs.root, NULL, NULL,
 				buf, NULL, 0))
@@ -609,32 +612,32 @@ err:
 	free_dist_actions(&actions);
 	if (ret) {
 		/* Kill environment */
-		logger(-1, 0, "VE start failed");
+		logger(-1, 0, "Container start failed");
 		write(wait_p[1], &err, sizeof(err));
 	} else {
 		if (!read(err_p[0], &ret, sizeof(ret))) {
 			if (res->misc.wait == YES) {
-				logger(0, 0, "VE start in progress"
+				logger(0, 0, "Container start in progress"
 					", waiting ...");
 				err = vps_execFn(h, veid, res->fs.root,
 					wait_on_fifo, NULL, 0);
 				if (err) {
-					logger(0, 0, "VE wait failed %s",
+					logger(0, 0, "Container wait failed%s",
 						err == VZ_EXEC_TIMEOUT ? \
-						"- timeout expired" : "");
+						" - timeout expired" : "");
 					ret = VZ_WAIT_FAILED;
 				} else {
-					logger(0, 0, "VE started"
+					logger(0, 0, "Container started"
 						" successfully");
 				}
 			} else {
-				logger(0, 0, "VE start in progress...");
+				logger(0, 0, "Container start in progress...");
 			}
 		} else {
 			if (ret == VZ_FS_BAD_TMPL)
 				logger(-1, 0, "Unable to start init, probably"
 					" incorrect template");
-			logger(-1, 0, "VE start failed");
+			logger(-1, 0, "Container start failed");
 		}
 	}
 	if (ret) {
@@ -653,14 +656,13 @@ err:
 	return ret;
 }
 
-/** Start and configure VE.
+/** Start and configure CT.
  *
- * @param h		VE handler.
- * @param veid		VE id.
- * @param res		VE resourses.
- * @param d_actions	distribution specific actions.
- * @param skip		flags to skip VE setup (SKIP_SETUP|SKIP_ACTION_SCRIPT)
- * @param action	modules list, used to call setup() callback
+ * @param h		CT handler.
+ * @param veid		CT ID.
+ * @param param		CT parameters.
+ * @param skip		flags to skip CT setup (SKIP_SETUP|SKIP_ACTION_SCRIPT)
+ * @param mod		modules list, used to call setup() callback
  * @return		0 on success.
  */
 int vps_start(vps_handler *h, envid_t veid, vps_param *param,
@@ -669,7 +671,7 @@ int vps_start(vps_handler *h, envid_t veid, vps_param *param,
 	return vps_start_custom(h, veid, param, skip, mod, NULL, NULL);
 }
 
-static int real_env_stop(vps_handler *h, envid_t veid, char *vps_root,
+static int real_env_stop(vps_handler *h, envid_t veid, const char *vps_root,
 	int stop_mode)
 {
 	int ret;
@@ -682,7 +684,7 @@ static int real_env_stop(vps_handler *h, envid_t veid, char *vps_root,
 	if ((ret = vz_env_create_ioctl(h, veid, VE_ENTER)) < 0) {
 		if (errno == ESRCH)
 			return 0;
-		logger(-1, errno, "VE enter failed");
+		logger(-1, errno, "Container enter failed");
 		return ret;
 	}
 	close(h->vzfd);
@@ -710,7 +712,8 @@ static int real_env_stop(vps_handler *h, envid_t veid, char *vps_root,
 	return 0;
 }
 
-static int env_stop(vps_handler *h, envid_t veid, char *root, int stop_mode)
+static int env_stop(vps_handler *h, envid_t veid, const char *root,
+		int stop_mode)
 {
 	struct sigaction act, actold;
 	int i, pid, ret = 0;
@@ -721,7 +724,7 @@ static int env_stop(vps_handler *h, envid_t veid, char *root, int stop_mode)
 	act.sa_flags = SA_NOCLDSTOP;
 	sigaction(SIGCHLD, &act, NULL);
 
-	logger(0, 0, "Stopping VE ...");
+	logger(0, 0, "Stopping container ...");
 	if (stop_mode == M_KILL)
 		goto kill_vps;
 	if ((pid = fork()) < 0) {
@@ -745,12 +748,12 @@ kill_vps:
 		ret = VZ_RESOURCE_ERROR;
 		logger(-1, errno, "Can not fork");
 		goto err;
-	
+
 	} else if (pid == 0) {
 		ret = real_env_stop(h, veid, root, M_KILL);
 		exit(ret);
 	}
-	ret = VZ_STOP_ERROR;	
+	ret = VZ_STOP_ERROR;
 	for (i = 0; i < MAX_SHTD_TM; i++) {
 		usleep(500000);
 		if (!vps_is_run(h, veid)) {
@@ -760,19 +763,19 @@ kill_vps:
 	}
 out:
 	if (ret)
-		logger(-1, 0, "Unable to stop VE: operation timed out");
+		logger(-1, 0, "Unable to stop container: operation timed out");
 	else
-		logger(0, 0, "VE was stopped");
+		logger(0, 0, "Container was stopped");
 err:
 	sigaction(SIGCHLD, &actold, NULL);
 	return ret;
 }
 
-/** Stop VE.
+/** Stop CT.
  *
- * @param h		VE handler.
- * @param veid		VE id.
- * @param res		VE resourses.
+ * @param h		CT handler.
+ * @param veid		CT ID.
+ * @param param		CT parameters.
  * @param stop_mode	stop mode, one of (M_REBOOT M_HALT M_KILL).
  * @param skip		flag to skip run action script (SKIP_ACTION_SCRIPT)
  * @param action	modules list, used to call cleanup() callback.
@@ -788,12 +791,12 @@ int vps_stop(vps_handler *h, envid_t veid, vps_param *param, int stop_mode,
 	if (check_var(res->fs.root, "VE_ROOT is not set"))
 		return VZ_VE_ROOT_NOTSET;
 	if (!vps_is_run(h, veid)) {
-		logger(-1, 0, "Unable to stop: VE is not running");
+		logger(-1, 0, "Unable to stop: container is not running");
 		return 0;
 	}
 	if (!(skip & SKIP_ACTION_SCRIPT)) {
 		snprintf(buf, sizeof(buf), VPS_CONF_DIR "%d.%s", veid,
-			STOP_PREFIX);	
+			STOP_PREFIX);
 		if (stat_file(buf)) {
 			if (vps_exec_script(h, veid, res->fs.root, NULL, NULL,
 				buf, NULL, 0))
@@ -802,32 +805,17 @@ int vps_stop(vps_handler *h, envid_t veid, vps_param *param, int stop_mode,
 			}
 		}
 	}
-	/* get VE IP addresses for cleanup */
+	/* get CT IP addresses for cleanup */
 	get_vps_ip(h, veid, &param->del_res.net.ip);
 	if ((ret = env_stop(h, veid, res->fs.root, stop_mode)))
 		goto end;
 	mod_cleanup(h, veid, action, param);
 	vps_cleanup_res(h, veid, param, STATE_STOPPING);
 	ret = vps_umount(h, veid, res->fs.root, skip);
-	/* Clear VE network configuration */
+	/* Clear CT network configuration */
 	ret = run_pre_script(veid, VPS_STOP);
 
 end:
 	free_str_param(&param->del_res.net.ip);
 	return ret;
 }
-
-int vps_restart(vps_handler *h, envid_t veid, vps_param *param)
-{
-	int ret;
-
-	logger(0, 0, "Restarting VE");
-	if (vps_is_run(h, veid) &&
-		(ret = vps_stop(h, veid, param, M_REBOOT, SKIP_NONE, NULL)))
-	{
-		return ret;
-	}
-	ret = vps_start(h, veid, param, SKIP_NONE, NULL);
-
-	return ret;
-}
