@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2008, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2009, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -78,8 +78,8 @@ int vps_meminfo_set(vps_handler *h, envid_t veid, meminfo_param *gparam,
 		break;
 	case VE_MEMINFO_PRIVVMPAGES:
 		if (privvmpages == NULL) {
-			logger(0, 0, "Warning: privvmpages is not set"
-				" configure meminfo skipped");
+			logger(0, 0, "Warning: privvmpages not set, "
+				"skipping meminfo configuration");
 			return 0;
 		}
 		meminfo.val = (((ULONG_MAX - 1) / param->val + 1)
@@ -91,10 +91,7 @@ int vps_meminfo_set(vps_handler *h, envid_t veid, meminfo_param *gparam,
 			" to set meminfo parameter");
 		return 0;
 	}
-	if (param->mode == VE_MEMINFO_NONE)
-		logger(0, 0, "Configure meminfo: none");
-	else
-		logger(0, 0, "Configure meminfo: %lu", meminfo.val);
+	logger(1, 0, "Configuring meminfo: %lu", meminfo.val);
 	ret = ioctl(h->vzfd, VZCTL_VE_MEMINFO, &meminfo);
 	if (ret < 0) {
 		if (errno == ENOTTY) {
@@ -110,7 +107,7 @@ int vps_meminfo_set(vps_handler *h, envid_t veid, meminfo_param *gparam,
 
 int get_meminfo_mode(char *name)
 {
-	int i;
+	unsigned int i;
 
 	for (i = 0; i < sizeof(mode_tbl_) / sizeof(mode_tbl_[0]); i++)
 		if (!strcmp(mode_tbl_[i].mode_nm, name))
@@ -121,7 +118,7 @@ int get_meminfo_mode(char *name)
 
 const char *get_meminfo_mode_nm(int id)
 {
-	int i;
+	unsigned int i;
 
 	for (i = 0; i < sizeof(mode_tbl_) / sizeof(mode_tbl_[0]); i++)
 		if (mode_tbl_[i].mode_id == id)
