@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2007 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2009, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,6 +23,10 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <signal.h>
+#include <unistd.h>
+#include <sys/types.h>
+#include <sys/wait.h>
 #include <linux/vzcalluser.h>
 #include <linux/vzctl_venet.h>
 
@@ -184,7 +188,7 @@ static int vps_ip_ctl(vps_handler *h, envid_t veid, int op,
 	return ret;
 }
 
-static inline int vps_add_ip(vps_handler *h, envid_t veid,
+int vps_add_ip(vps_handler *h, envid_t veid,
 	net_param *net, int state)
 {
 	char *str;
@@ -192,7 +196,8 @@ static inline int vps_add_ip(vps_handler *h, envid_t veid,
 	list_head_t *ip_h = &net->ip;
 
 	if ((str = list2str(NULL, ip_h)) != NULL) {
-		logger(0, 0, "Adding IP address(es): %s", str);
+		if (str[0] != '\0')
+			logger(0, 0, "Adding IP address(es): %s", str);
 		free(str);
 	}
 	if ((ret = vps_ip_ctl(h, veid, VE_IP_ADD, ip_h, 1)))
@@ -203,7 +208,7 @@ static inline int vps_add_ip(vps_handler *h, envid_t veid,
 	return ret;
 }
 
-static inline int vps_del_ip(vps_handler *h, envid_t veid,
+int vps_del_ip(vps_handler *h, envid_t veid,
 	net_param *net, int state)
 {
 	char *str;
@@ -211,7 +216,8 @@ static inline int vps_del_ip(vps_handler *h, envid_t veid,
 	list_head_t *ip_h = &net->ip;
 
 	if ((str = list2str(NULL, ip_h)) != NULL) {
-		logger(0, 0, "Deleting IP address(es): %s", str);
+		if (str[0] != '\0')
+			logger(0, 0, "Deleting IP address(es): %s", str);
 		free(str);
 	}
 	if ((ret = vps_ip_ctl(h, veid, VE_IP_DEL, ip_h, 1)))
@@ -221,7 +227,7 @@ static inline int vps_del_ip(vps_handler *h, envid_t veid,
 	return ret;
 }
 
-static inline int vps_set_ip(vps_handler *h, envid_t veid,
+int vps_set_ip(vps_handler *h, envid_t veid,
 	net_param *net, int state)
 {
 	int ret;
@@ -252,31 +258,98 @@ static int netdev_ctl(vps_handler *h, int veid, int op, char *name)
 	return 0;
 }
 
-int vps_netdev_ctl(vps_handler *h, envid_t veid, int op, net_param *net)
+int set_netdev(vps_handler *h, envid_t veid, int cmd, net_param *net)
 {
 	int ret = 0;
 	list_head_t *dev_h = &net->dev;
 	net_dev_param *dev;
-	int cmd;
 
 	if (list_empty(dev_h))
 		return 0;
-	if (!vps_is_run(h, veid)){
-		logger(-1, 0, "Unable to setup network devices: "
-			"VE is not running");
-		return VZ_VE_NOT_RUNNING;
-	}
-	cmd = (op == ADD) ? VE_NETDEV_ADD : VE_NETDEV_DEL;
 	list_for_each(dev, dev_h, list) {
 		if ((ret = netdev_ctl(h, veid, cmd, dev->val))) {
 			logger(-1, errno, "Unable to %s netdev %s",
-				(op == ADD) ? "add": "del", dev->val);
+				(cmd == VE_NETDEV_ADD ) ? "add": "del",
+				dev->val);
 			break;
 		}
 	}
 	return ret;
 }
 
+int vps_set_netdev(vps_handler *h, envid_t veid, ub_param *ub,
+		net_param *net_add, net_param *net_del)
+{
+	struct sigaction act;
+	int ret, pid, pid1, status;
+
+	if (list_empty(&net_add->dev) && list_empty(&net_del->dev))
+		return 0;
+	if (!vps_is_run(h, veid)){
+		logger(-1, 0, "Unable to setup network devices: "
+			"container is not running");
+		return VZ_VE_NOT_RUNNING;
+	}
+
+	sigemptyset(&act.sa_mask);
+	act.sa_handler = SIG_DFL;
+	act.sa_flags = SA_NOCLDSTOP;
+	sigaction(SIGCHLD, &act, NULL);
+
+	if ((pid1 = fork()) < 0) {
+		logger(0, errno, "Can't fork");
+		return VZ_RESOURCE_ERROR;
+	} else if (pid1 == 0) {
+		int pid2;
+
+		if ((ret = vz_setluid(veid))) {
+			exit(ret);
+		}
+		if ((ret = vps_set_ublimit(h, veid, ub))) {
+			exit(ret);
+		}
+		/* Create another process for proper accounting */
+		if ((pid2 = fork()) < 0) {
+			logger(0, errno, "Can't fork");
+			exit(VZ_RESOURCE_ERROR);
+		} else if (pid2 == 0) {
+			if ((ret = set_netdev(h, veid,
+					VE_NETDEV_DEL, net_del)))
+				exit(ret);
+			ret = set_netdev(h, veid, VE_NETDEV_ADD, net_add);
+			exit(ret);
+		}
+		while ((pid = waitpid(pid2, &status, 0)) == -1)
+			if (errno != EINTR)
+				break;
+		ret = VZ_SYSTEM_ERROR;
+		if (pid == pid2) {
+			if (WIFEXITED(status))
+				ret = WEXITSTATUS(status);
+			else if (WIFSIGNALED(status))
+				logger(0, 0, "Got signal %d",
+						WTERMSIG(status));
+			} else if (pid < 0)
+				logger(0, errno, "Error in waitpid()");
+		exit(ret);
+		}
+	while ((pid = waitpid(pid1, &status, 0)) == -1)
+		if (errno != EINTR) {
+			logger(0, errno, "Error in waitpid()");
+			break;
+		}
+	ret = VZ_SYSTEM_ERROR;
+	if (pid == pid1) {
+		if (WIFEXITED(status))
+			ret = WEXITSTATUS(status);
+		else if (WIFSIGNALED(status))
+			logger(0, 0, "Got signal %d", WTERMSIG(status));
+	} else if (pid < 0)
+		logger(0, errno, "Error in waitpid()");
+
+	return ret;
+}
+
 static int remove_ipv6_addr(net_param *net)
 {
 	list_head_t *head = &net->ip;
@@ -296,15 +369,16 @@ static int remove_ipv6_addr(net_param *net)
 }
 
 int vps_net_ctl(vps_handler *h, envid_t veid, int op, net_param *net,
-	dist_actions *actions, char *root, int state, int skip)
+	dist_actions *actions, const char *root, int state, int skip)
 {
 	int ret = 0;
 
-	if (list_empty(&net->ip) && !net->delall)
+	if (list_empty(&net->ip) && !net->delall &&
+			!(state == STATE_STARTING && op == ADD))
 		return 0;
 	if (!vps_is_run(h, veid)) {
 		logger(-1, 0, "Unable to apply network parameters: "
-			"VE is not running");
+			"container is not running");
 		return VZ_VE_NOT_RUNNING;
 	}
 	if (net->ipv6_net != YES) {
@@ -325,7 +399,7 @@ int vps_net_ctl(vps_handler *h, envid_t veid, int op, net_param *net,
 }
 
 #define	PROC_VEINFO	"/proc/vz/veinfo"
-static inline int get_vps_ip_proc(envid_t veid, list_head_t *ip_h)
+int get_vps_ip_proc(envid_t veid, list_head_t *ip_h)
 {
 	FILE *fd;
 	char str[16384];
@@ -378,7 +452,7 @@ static inline int get_vps_ip_proc(envid_t veid, list_head_t *ip_h)
 	((unsigned char *)&addr)[3]
 #endif
 
-static inline int get_vps_ip_ioctl(vps_handler *h, envid_t veid,
+int get_vps_ip_ioctl(vps_handler *h, envid_t veid,
 	list_head_t *ip_h)
 {
 	int ret = -1;
@@ -441,4 +515,3 @@ int get_vps_ip(vps_handler *h, envid_t veid, list_head_t *ip_h)
 	return ret;
 }
 #endif
-
