@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2007 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2009, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@
 #include "fs.h"
 
 volatile sig_atomic_t alarm_flag;
-static char *envp_bash[] = {"HOME=/", "TERM=linux", "PATH=/bin:/sbin:/usr/bin:/usr/sbin:", NULL};
+static char *envp_bash[] = {"HOME=/", "TERM=linux", ENV_PATH, NULL};
 
 int read_script(const char *fname, char *include, char **buf)
 {
@@ -53,7 +53,9 @@ int read_script(const char *fname, char *include, char **buf)
 	}
 	/* Read include file first */
 	if (include != NULL) {
-		inc = malloc(strlen(fname) + strlen(include) + 1);
+		inc = vz_malloc(strlen(fname) + strlen(include) + 1);
+		if (!inc)
+			return -1;
 		if ((p = strrchr(fname, '/')) != NULL) {
 			snprintf(inc, p - fname + 2, "%s", fname);
 			strcat(inc, include);
@@ -62,7 +64,7 @@ int read_script(const char *fname, char *include, char **buf)
 		}
 		if (stat_file(inc))
 			len = read_script(inc, NULL, buf);
-		if (inc != NULL) free(inc);
+		free(inc);
 		if (len < 0)
 			return -1;
 	}
@@ -206,12 +208,7 @@ int run_pre_script(int veid, char *script)
 	env[1] = strdup(buf);
 	env[2] = strdup(ENV_PATH);
 	env[3] = NULL;
-/*
-	if ((dist = get_ostmpl(gparam)) != NULL) {
-		snprintf(buf, sizeof(buf), "DIST=%s", dist);
-		env = ListAddElem(env, buf, NULL, NULL);
-	}
-*/
+
 	if ((ret = run_script(script, arg, env, 0)))
 		ret = VZ_ACTIONSCRIPT_ERROR;
 	free_arg(env);
@@ -227,7 +224,7 @@ int mk_reboot_script()
 #define VZREBOOT	"S00vzreboot"
 #define RC1		"/etc/rc.d/rc6.d"
 #define RC2		"/etc/rc6.d"
-#define REBOOT_SCRIPT	"#!/bin/bash\n>/" REBOOT_MARK
+#define REBOOT_SCRIPT	"#!/bin/bash\n>" REBOOT_MARK
 
 	/* remove reboot flag */
 	unlink(REBOOT_MARK);
@@ -243,7 +240,7 @@ int mk_reboot_script()
 		logger(-1, errno, "Unable to create %s", buf);
 		return 1;
 	}
-	write(fd, REBOOT_SCRIPT, sizeof(REBOOT_SCRIPT));
+	write(fd, REBOOT_SCRIPT, sizeof(REBOOT_SCRIPT) - 1);
 	close(fd);
 
 	return 0;
@@ -284,13 +281,42 @@ int mk_quota_link()
 
 #define INITTAB_FILE		"/etc/inittab"
 #define INITTAB_VZID		"vz:"
-#define INITTAB_ACTION		INITTAB_VZID "35:once:touch " VZFIFO_FILE "\n"
+#define INITTAB_ACTION		INITTAB_VZID "2345:once:touch " VZFIFO_FILE "\n"
+
+#define EVENTS_DIR		"/etc/event.d/"
+#define EVENTS_FILE		EVENTS_DIR "vz_init_done"
+#define EVENTS_SCRIPT	\
+	"# This task runs if default runlevel is reached\n"	\
+	"# Added by OpenVZ vzctl\n"				\
+	"start on stopped rc2\n"				\
+	"start on stopped rc3\n"				\
+	"start on stopped rc4\n"				\
+	"start on stopped rc5\n"				\
+	"exec touch " VZFIFO_FILE "\n"
+
+/* Ubuntu 9.10 needs different machinery */
+#define EVENTS_DIR_UBUNTU	"/etc/init/"
+#define EVENTS_FILE_UBUNTU	EVENTS_DIR_UBUNTU "vz_init_done.conf"
+#define EVENTS_SCRIPT_UBUNTU	\
+	"# tell vzctl that start was successfull\n"			\
+	"#\n"								\
+	"# This task is to tell vzctl that start was successfull\n"	\
+	"\n"								\
+	"description	\"tell vzctl that start was successfull\"\n"	\
+	"\n"								\
+	"start on stopped rc RUNLEVEL=[2345]\n"				\
+	"\n"								\
+	"task\n"							\
+	"\n"								\
+	"exec touch " VZFIFO_FILE "\n"
+
 #define MAX_WAIT_TIMEOUT	60 * 60
 
 int add_reach_runlevel_mark()
 {
 	int fd, found, len, ret;
 	char buf[4096];
+	struct stat st;
 
 	unlink(VZFIFO_FILE);
 	if (mkfifo(VZFIFO_FILE, 0644)) {
@@ -298,6 +324,31 @@ int add_reach_runlevel_mark()
 			strerror(errno));
 		return -1;
 	}
+	/* Create upstart specific script */
+	if (!stat(EVENTS_DIR, &st)) {
+		fd = open(EVENTS_FILE, O_WRONLY|O_TRUNC|O_CREAT, 0644);
+		if (fd == -1) {
+			fprintf(stderr, "Unable to create " EVENTS_FILE
+				": %s\n", strerror(errno));
+			return -1;
+		}
+		write(fd, EVENTS_SCRIPT, sizeof(EVENTS_SCRIPT) - 1);
+		close(fd);
+		return 0;
+	} else if (!stat(EVENTS_DIR_UBUNTU, &st)) {
+		fd = open(EVENTS_FILE_UBUNTU, O_WRONLY|O_TRUNC|O_CREAT, 0644);
+		if (fd == -1) {
+			fprintf(stderr, "Unable to create "
+				EVENTS_FILE_UBUNTU ": %s\n",
+				strerror(errno));
+			return -1;
+		}
+		write(fd, EVENTS_SCRIPT_UBUNTU,
+			sizeof(EVENTS_SCRIPT_UBUNTU) - 1);
+		close(fd);
+		return 0;
+	}
+	/* Add a line to /etc/inittab */
 	if ((fd = open(INITTAB_FILE, O_RDWR | O_APPEND)) == -1) {
 		fprintf(stderr, "Unable to open " INITTAB_FILE " %s\n",
 			strerror(errno));
@@ -370,4 +421,3 @@ err:
 	unlink(VZFIFO_FILE);
 	return ret;
 }
-
